@@ -9,48 +9,24 @@
 
 #include <config.h>
 
-#include "ggv-window.h"
 #include "ggv-prefs.h"
 #include "ggvutils.h"
 #include "gsdefaults.h"
 
 static GConfClient *gconf_client = NULL;
-static GSList *cb_list = NULL;
 
 gchar    *ggv_print_cmd = NULL;
 gboolean  ggv_save_geometry = FALSE;
 gint      ggv_default_width = DEFAULT_WINDOW_WIDTH;
-gint      file_sel_width = DEFAULT_FILE_SEL_WIDTH;
 gint      ggv_default_height = DEFAULT_WINDOW_HEIGHT;
-gint      file_sel_height = DEFAULT_FILE_SEL_HEIGHT;
 
 gboolean ggv_panel = TRUE, ggv_menubar = TRUE, ggv_toolbar = TRUE;
-gboolean ggv_statusbar = TRUE, ggv_autojump = TRUE;
+gboolean ggv_statusbar = TRUE, ggv_autojump = TRUE, ggv_pageflip = FALSE;
 GgvToolbarStyle ggv_toolbar_style = GGV_TOOLBAR_STYLE_DEFAULT;
 gboolean ggv_right_panel = FALSE;
 
 gint ggv_unit_index = 0;
 
-typedef struct _GgvPrefsCbData
-{
-        gpointer data;
-        GgvPrefsCb cb;
-} GgvPrefsCbData;
-
-static void
-ggv_prefs_execute_callbacks()
-{
-        GSList *node;
-        GgvPrefsCbData *cbd;
-
-        node = cb_list;
-        while(node) {
-                cbd = (GgvPrefsCbData *)node->data;
-                node = node->next;
-                cbd->cb(cbd->data);
-        }
-}
-
 void
 ggv_prefs_load()
 {
@@ -87,6 +63,9 @@ ggv_prefs_load()
         ggv_autojump = gconf_client_get_bool(gconf_client,
                                               "/apps/ggv/control/autojump",
                                               NULL);
+        ggv_pageflip = gconf_client_get_bool(gconf_client,
+                                             "/apps/ggv/control/pageflip",
+                                             NULL);
         ggv_right_panel = gconf_client_get_bool(gconf_client,
                                                "/apps/ggv/layout/rightpanel",
                                                NULL);
@@ -99,14 +78,6 @@ ggv_prefs_load()
         if((ggv_default_height = gconf_client_get_int
             (gconf_client, "/apps/ggv/layout/windowheight", NULL)) == 0)
                 ggv_default_height = DEFAULT_WINDOW_HEIGHT;
-        if((file_sel_width = gconf_client_get_int
-            (gconf_client, "/apps/ggv/layout/fileselwidth", NULL)) == 0)
-                file_sel_width = DEFAULT_FILE_SEL_WIDTH;
-        if((file_sel_height = gconf_client_get_int
-            (gconf_client, "/apps/ggv/layout/fileselheight", NULL)) == 0)
-                file_sel_height = DEFAULT_FILE_SEL_WIDTH;
-
-        ggv_prefs_execute_callbacks();
 }
 
 void
@@ -132,6 +103,8 @@ ggv_prefs_save()
                               ggv_save_geometry, NULL);
         gconf_client_set_bool(gconf_client, "/apps/ggv/control/autojump",
                               ggv_autojump, NULL);
+        gconf_client_set_bool(gconf_client, "/apps/ggv/control/pageflip",
+                              ggv_pageflip, NULL);
         gconf_client_set_bool(gconf_client, "/apps/ggv/layout/rightpanel",
                               ggv_right_panel, NULL);
         if(ggv_save_geometry) {
@@ -139,10 +112,6 @@ ggv_prefs_save()
                                      ggv_default_width, NULL);
                 gconf_client_set_int(gconf_client, "/apps/ggv/layout/windowheight",
                                      ggv_default_height, NULL);
-                gconf_client_set_int(gconf_client, "/apps/ggv/layout/fileselwidth",
-                                     file_sel_width, NULL);
-                gconf_client_set_int(gconf_client, "/apps/ggv/layout/fileselheight",
-                                     file_sel_height, NULL);
         }
 }
 
@@ -172,7 +141,7 @@ ggv_prefs_changed(GConfClient* client, guint cnxn_id,
                                                     NULL);
         }
         else if(!strcmp(entry->key, "/apps/ggv/layout/showstatusbar")) {
-                ggv_toolbar = gconf_client_get_bool(gconf_client,
+                ggv_statusbar = gconf_client_get_bool(gconf_client,
                                                     "/apps/ggv/layout/showstatusbar",
                                                     NULL);
         }
@@ -191,6 +160,11 @@ ggv_prefs_changed(GConfClient* client, guint cnxn_id,
                                                      "/apps/ggv/control/autojump",
                                                      NULL);
         }
+        else if(!strcmp(entry->key, "/apps/ggv/control/pageflip")) {
+                ggv_pageflip = gconf_client_get_bool(gconf_client,
+                                                     "/apps/ggv/control/pageflip",
+                                                     NULL);
+        }
         else if(!strcmp(entry->key, "/apps/ggv/layout/rightpanel")) {
                 ggv_right_panel = gconf_client_get_bool(gconf_client,
                                                        "/apps/ggv/layout/rightpanel",
@@ -210,18 +184,6 @@ ggv_prefs_changed(GConfClient* client, guint cnxn_id,
                     (gconf_client, "/apps/ggv/layout/windowheight", NULL)) == 0)
                         ggv_default_height = DEFAULT_WINDOW_HEIGHT;
         }
-        else if(!strcmp(entry->key, "/apps/ggv/layout/fileselwidth")) {
-                if((file_sel_width = gconf_client_get_int
-                    (gconf_client, "/apps/ggv/layout/fileselwidth", NULL)) == 0)
-                        file_sel_width = DEFAULT_FILE_SEL_WIDTH;
-        }
-        else if(!strcmp(entry->key, "/apps/ggv/layout/fileselheight")) {
-                if((file_sel_height = gconf_client_get_int
-                    (gconf_client, "/apps/ggv/layout/fileselheight", NULL)) == 0)
-                        file_sel_height = DEFAULT_FILE_SEL_WIDTH;
-        }
-
-        ggv_prefs_execute_callbacks();
 }
 
 GConfClient *
@@ -240,35 +202,3 @@ ggv_prefs_gconf_client()
 
         return gconf_client;
 }
-
-void
-ggv_prefs_add_callback(GgvPrefsCb cb, gpointer data)
-{
-        GgvPrefsCbData *cbd;
-
-        cbd = g_new0(GgvPrefsCbData, 1);
-        cbd->cb = cb;
-        cbd->data = data;
-        cb_list = g_slist_append(cb_list, cbd);
-}
-
-void
-ggv_prefs_remove_callback(gpointer data)
-{
-        GSList *node;
-        GgvPrefsCbData *cbd;
-
-        node = cb_list;
-        while(node) {
-                cbd = (GgvPrefsCbData *)node->data;
-                if(cbd->data == data)
-                        break;
-                else
-                        cbd = NULL;
-                node = node->next;
-        }
-        if(cbd) {
-                cb_list = g_slist_remove(cb_list, cbd);
-                g_free(cbd);
-        }
-}
