@@ -40,14 +40,18 @@ extern gboolean  ggv_statusbar;       /* statusbar visible */
 extern gboolean  ggv_save_geometry;   /* Save the current geometry for next session */
 extern gint      ggv_unit_index;      /* the unit we want to use for coordinates */
 extern gboolean  ggv_autojump;        /* auto jump to beginning of the page */
+extern gboolean  ggv_pageflip;        /* automatically flip pages */
 extern gboolean  ggv_right_panel;     /* panel appears on the left side by default */
 
 /* default window dimensions */
-extern gint ggv_default_width, file_sel_width;
-extern gint ggv_default_height, file_sel_height;
+extern gint ggv_default_width;
+extern gint ggv_default_height;
 
+/* this is used _only_ by the ggv-postscript-viewer */
 void         ggv_prefs_load (void);
 void         ggv_prefs_save (void);
+
+/* this is also used by ggv shell */
 GConfClient *ggv_prefs_gconf_client (void);
 
 G_END_DECLS
