@@ -14,6 +14,10 @@
 #include <libgnomeui/gnome-window-icon.h>
 #include <bonobo.h>
 #include <bonobo/bonobo-ui-main.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
+#include <X11/Xlib.h>
+#include <X11/Xatom.h> 
+#include <gdk/gdkx.h>
 
 #include <math.h>
 #include <ctype.h>
@@ -22,18 +26,230 @@
 #include "gtkchecklist.h"
 #include "ggv-prefs.h"
 #include "ggv-window.h"
+#include "ggv-file-sel.h"
+#include "ggvutils.h"
 
 #define GGV_POSTSCRIPT_VIEWER_CONTROL_IID "OAFIID:GNOME_GGV_Control"
 
 static BonoboWindowClass *parent_class;
 
-static GList *window_list;
+static GList *window_list = NULL;
 
 /* what can be dragged in us... */
 enum {
         TARGET_URI_LIST,
 };
 
+static void
+sync_settings_menu_items(GgvWindow *win)
+{
+        if(win->uic == NULL)
+                return;
+
+	bonobo_ui_component_freeze(win->uic, NULL);
+        bonobo_ui_component_set_prop(win->uic, "/commands/SettingsShowMenus", "state",
+                                     win->show_menus?"1":"0", NULL);
+        bonobo_ui_component_set_prop(win->uic, "/commands/SettingsShowToolbar", "state",
+                                     win->show_toolbar?"1":"0", NULL);
+        bonobo_ui_component_set_prop(win->uic, "/commands/SettingsShowSidebar", "state",
+                                     win->show_sidebar?"1":"0", NULL);
+        bonobo_ui_component_set_prop(win->uic, "/commands/SettingsShowStatusbar", "state",
+                                     win->show_statusbar?"1":"0", NULL);
+	bonobo_ui_component_thaw(win->uic, NULL);
+}
+
+static void
+sync_settings_popup_items(GgvWindow *win)
+{
+        if(win->popup_uic == NULL)
+                return;
+
+	bonobo_ui_component_freeze(win->popup_uic, NULL);
+        bonobo_ui_component_set_prop(win->popup_uic, "/commands/SettingsShowMenus", "state",
+                                     win->show_menus?"1":"0", NULL);
+        bonobo_ui_component_set_prop(win->popup_uic, "/commands/SettingsShowToolbar", "state",
+                                     win->show_toolbar?"1":"0", NULL);
+        bonobo_ui_component_set_prop(win->popup_uic, "/commands/SettingsShowSidebar", "state",
+                                     win->show_sidebar?"1":"0", NULL);
+        bonobo_ui_component_set_prop(win->popup_uic, "/commands/SettingsShowStatusbar", "state",
+                                     win->show_statusbar?"1":"0", NULL);
+	bonobo_ui_component_thaw(win->popup_uic, NULL);
+}
+
+void
+sync_toolbar_style(GgvWindow *win)
+{
+        const gchar *tbstyle;
+        gchar *gnome_tbstyle;
+
+        switch(ggv_toolbar_style) {
+        case GGV_TOOLBAR_STYLE_DEFAULT:
+                gnome_tbstyle = gconf_client_get_string(ggv_prefs_gconf_client(),
+                                                        "/desktop/gnome/interface/toolbar_style",
+                                                        NULL);
+                if(gnome_tbstyle) {
+                        if(!strcmp(gnome_tbstyle, "icons"))
+                                tbstyle = "icon";
+                        else if(!strcmp(gnome_tbstyle, "both"))
+                                tbstyle = "both";
+                        else if(!strcmp(gnome_tbstyle, "text"))
+                                tbstyle = "text";
+                        else if(!strcmp(gnome_tbstyle, "both_horiz"))
+                                tbstyle = "both_horiz";
+                        else
+                                tbstyle = "both";
+                        g_free(gnome_tbstyle);
+                }
+                else
+                        tbstyle = "both";
+                break;
+        case GGV_TOOLBAR_STYLE_BOTH:
+                tbstyle = "both";
+                break;
+        case GGV_TOOLBAR_STYLE_ICONS:
+                tbstyle = "icon";
+                break;
+        case GGV_TOOLBAR_STYLE_TEXT:
+                tbstyle = "text";
+                break;
+        default:
+                tbstyle = NULL;
+                break;
+        }
+        if(tbstyle)
+                bonobo_ui_component_set_prop(win->uic, "/Toolbar", "look",
+                                             tbstyle, NULL);
+}
+
+static void
+sync_fullscreen_items(GgvWindow *win)
+{
+        if(win->uic != NULL) {
+                bonobo_ui_component_set_prop(win->uic, "/commands/ViewFullscreen", "state",
+                                             win->fullscreen?"1":"0", NULL);
+        }
+        if(win->popup_uic != NULL) {
+                bonobo_ui_component_set_prop(win->popup_uic, "/commands/ViewFullscreen", "state",
+                                             win->fullscreen?"1":"0", NULL);
+        }
+}
+
+/* utility functions to raise window above all others! */
+#define XA_WIN_LAYER         "_WIN_LAYER"
+typedef enum {
+        WIN_LAYER_NORMAL = 4,
+        WIN_LAYER_ABOVE_DOCK = 10
+} GgvWindowLayer;
+static Atom A_XA_WIN_LAYER;
+static gboolean atom_initialized = FALSE;
+
+static void
+ggv_init_atoms()
+{
+        if(!GDK_DISPLAY() || atom_initialized) return;
+        A_XA_WIN_LAYER = XInternAtom(GDK_DISPLAY(), XA_WIN_LAYER, False);
+        atom_initialized = TRUE;
+}
+
+static void
+ggv_window_set_layer(GgvWindow *win, GgvWindowLayer layer)
+{
+        if(!atom_initialized)
+                ggv_init_atoms();
+        if(GTK_WIDGET_MAPPED(win)) {
+                XEvent e;
+                e.type = ClientMessage;
+                e.xclient.type = ClientMessage;
+                e.xclient.window = GDK_WINDOW_XID(GTK_WIDGET(win)->window);
+                e.xclient.message_type = A_XA_WIN_LAYER;
+                e.xclient.format = 32;
+                e.xclient.data.l[0] = (long)layer;
+                e.xclient.data.l[1] = GDK_CURRENT_TIME;
+
+                XSendEvent(GDK_DISPLAY(), GDK_ROOT_WINDOW(), False,
+                           SubstructureNotifyMask, (XEvent *)&e);
+        }
+        else {
+                long data[1];
+
+                data[0] = layer;
+                XChangeProperty(GDK_DISPLAY(),
+                                GDK_WINDOW_XID(GTK_WIDGET(win)->window),
+                                A_XA_WIN_LAYER, XA_CARDINAL, 32,
+                                PropModeReplace, (unsigned char *)data, 1);
+        }
+}
+
+static void
+ggv_window_set_fullscreen(GgvWindow *win, gboolean fs)
+{
+        if(win->fullscreen == fs)
+                return;
+
+        win->fullscreen = fs;
+        if(win->fullscreen) {
+                gint clx, cly, rx, ry, w, h;
+                gdk_window_get_origin(GTK_WIDGET(win)->window, &rx, &ry);
+                gdk_window_get_geometry(GTK_WIDGET(win)->window,
+                                        &clx, &cly, &w, &h, NULL);
+                win->orig_x = rx - clx;
+                win->orig_y = ry - cly;
+                win->orig_width = w;
+                win->orig_height = h;
+                if(win->show_menus) {
+                        bonobo_ui_component_set_prop(win->uic, "/menu",
+                                                     "hidden", "1", NULL);
+                }
+                if(win->show_toolbar) {
+                        bonobo_ui_component_set_prop(win->uic, "/Toolbar",
+                                                     "hidden", "1", NULL);
+                }
+                if(win->show_statusbar) {
+                        gtk_widget_hide(win->statusbar);
+                }
+                if(win->show_sidebar) {
+                        bonobo_ui_component_set_prop(win->uic, "/Sidebar",
+                                                     "hidden", "1", NULL);
+                }
+                win->orig_sm = win->show_menus;
+                win->orig_st = win->show_toolbar;
+                win->orig_ss = win->show_sidebar;
+                win->orig_sss = win->show_statusbar;
+                win->show_menus = win->show_toolbar = win->show_sidebar = win->show_statusbar = FALSE;
+                gtk_window_move(GTK_WINDOW(win), -clx, -cly);
+                gtk_window_resize(GTK_WINDOW(win),
+                                  gdk_screen_width(),
+                                  gdk_screen_height());
+        }
+        else {
+                win->show_menus = win->orig_sm;
+                win->show_toolbar = win->orig_st;
+                win->show_sidebar = win->orig_ss;
+                win->show_statusbar = win->orig_sss;
+                if(win->show_menus) {
+                        bonobo_ui_component_set_prop(win->uic, "/menu",
+                                                     "hidden", "0", NULL);
+                }
+                if(win->show_toolbar) {
+                        bonobo_ui_component_set_prop(win->uic, "/Toolbar",
+                                                     "hidden", "0", NULL);
+                }
+                if(win->show_statusbar) {
+                        gtk_widget_show(win->statusbar);
+                }
+                if(win->show_sidebar) {
+                        bonobo_ui_component_set_prop(win->uic, "/Sidebar",
+                                                     "hidden", "0", NULL);
+                }
+                gtk_window_move(GTK_WINDOW(win), win->orig_x, win->orig_y);
+                gtk_window_resize(GTK_WINDOW(win), win->orig_width, win->orig_height);
+        }
+        ggv_window_set_layer(win, fs?WIN_LAYER_ABOVE_DOCK:WIN_LAYER_NORMAL);
+        sync_settings_menu_items(win);
+        sync_settings_popup_items(win);
+        sync_fullscreen_items(win);
+}
+
 static void
 ggv_window_drag_data_received(GtkWidget *widget,
                               GdkDragContext *context,
@@ -69,15 +285,16 @@ ggv_window_drag_data_received(GtkWidget *widget,
                         newwin = GTK_WIDGET(win);
                 else
                         newwin = NULL;
-                while(*uri) {
-                        if(!g_strncasecmp("file:", *uri, 5)) {
-                                if(newwin == NULL) {
-                                        newwin = ggv_window_new();
-                                        gtk_widget_show(newwin);
-                                }
-                                ggv_window_load(GGV_WINDOW(newwin), (*uri) + 5);
-                                newwin = NULL;
+                while(*uri && **uri != '\0') {
+#ifdef _DEBUG
+                        g_message("URI %s\n", *uri);
+#endif /* _DEBUG */
+                        if(newwin == NULL) {
+                                newwin = ggv_window_new();
+                                gtk_widget_show(newwin);
                         }
+                        ggv_window_load(GGV_WINDOW(newwin), (*uri));
+                        newwin = NULL;
                         uri++;
                 }
                 g_strfreev(win->uris_to_open);
@@ -85,78 +302,6 @@ ggv_window_drag_data_received(GtkWidget *widget,
         }
 }
 
-static void
-sync_settings_menu_items(GgvWindow *win)
-{
-        if(win->uic == NULL)
-                return;
-
-        bonobo_ui_component_set_prop(win->uic, "/commands/SettingsShowMenus", "state",
-                                     win->show_menus?"1":"0", NULL);
-        bonobo_ui_component_set_prop(win->uic, "/commands/SettingsShowToolbar", "state",
-                                     win->show_toolbar?"1":"0", NULL);
-        bonobo_ui_component_set_prop(win->uic, "/commands/SettingsShowSidebar", "state",
-                                     win->show_sidebar?"1":"0", NULL);
-        bonobo_ui_component_set_prop(win->uic, "/commands/SettingsShowStatusbar", "state",
-                                     win->show_statusbar?"1":"0", NULL);
-}
-
-static void
-sync_settings_popup_items(GgvWindow *win)
-{
-        if(win->popup_uic == NULL)
-                return;
-
-        bonobo_ui_component_set_prop(win->popup_uic, "/commands/SettingsShowMenus", "state",
-                                     win->show_menus?"1":"0", NULL);
-        bonobo_ui_component_set_prop(win->popup_uic, "/commands/SettingsShowToolbar", "state",
-                                     win->show_toolbar?"1":"0", NULL);
-        bonobo_ui_component_set_prop(win->popup_uic, "/commands/SettingsShowSidebar", "state",
-                                     win->show_sidebar?"1":"0", NULL);
-        bonobo_ui_component_set_prop(win->popup_uic, "/commands/SettingsShowStatusbar", "state",
-                                     win->show_statusbar?"1":"0", NULL);
-}
-
-static void
-sync_toolbar_style(GgvWindow *win)
-{
-        const gchar *tbstyle;
-        gchar *gnome_tbstyle;
-
-        switch(ggv_toolbar_style) {
-        case GGV_TOOLBAR_STYLE_DEFAULT:
-                gnome_tbstyle = gconf_client_get_string(ggv_prefs_gconf_client(),
-                                                        "/desktop/gnome/interface/toolbar_style",
-                                                        NULL);
-                if(gnome_tbstyle) {
-                        if(!strcmp(gnome_tbstyle, "icons"))
-                                tbstyle = "icon";
-                        else if(!strcmp(gnome_tbstyle, "both"))
-                                tbstyle = "both";
-                        else if(!strcmp(gnome_tbstyle, "text"))
-                                tbstyle = "text";
-                        else
-                                tbstyle = "both";
-                        g_free(gnome_tbstyle);
-                }
-                else
-                        tbstyle = "both";
-                break;
-        case GGV_TOOLBAR_STYLE_BOTH:
-                tbstyle = "both";
-                break;
-        case GGV_TOOLBAR_STYLE_ICONS:
-                tbstyle = "icon";
-                break;
-        case GGV_TOOLBAR_STYLE_TEXT:
-                tbstyle = "text";
-                break;
-        }
-        if(tbstyle)
-                bonobo_ui_component_set_prop(win->uic, "/Toolbar", "look",
-                                             tbstyle, NULL);
-}
-
 static void
 control_property_changed_handler(BonoboListener    *listener,
                                  char              *event_name, 
@@ -166,16 +311,13 @@ control_property_changed_handler(BonoboListener    *listener,
 {
         GgvWindow *win = GGV_WINDOW(data);
 
-        if(!g_strcasecmp(event_name, "Bonobo/Property:change:title")) {
-                gchar *title = g_strconcat(_("GGV: "), BONOBO_ARG_GET_STRING(any), NULL);
-                gtk_window_set_title(GTK_WINDOW(win), title);
-                g_free(title);
+        if(!g_ascii_strcasecmp(event_name, "Bonobo/Property:change:title")) {
         }
-        else if(!g_strcasecmp(event_name, "Bonobo/Property:change:status")) {
+        else if(!g_ascii_strcasecmp(event_name, "Bonobo/Property:change:status")) {
                 gnome_appbar_set_status(GNOME_APPBAR(win->statusbar),
                                         BONOBO_ARG_GET_STRING(any));
         }
-        else if(!g_strcasecmp(event_name, "Bonobo/Property:change:page")) {
+        else if(!g_ascii_strcasecmp(event_name, "Bonobo/Property:change:page")) {
                 if(win->current_page == BONOBO_ARG_GET_LONG(any) &&
                    win->current_page < 0)
                         return;
@@ -183,100 +325,67 @@ control_property_changed_handler(BonoboListener    *listener,
                 win->current_page = BONOBO_ARG_GET_LONG(any);
                 if(win->current_page < 0) {
                         GtkWidget *dlg;
-                        dlg = gtk_message_dialog_new(GTK_WINDOW(win),
-                                                     GTK_DIALOG_MODAL,
-                                                     GTK_MESSAGE_ERROR,
-                                                     GTK_BUTTONS_OK,
-                                                     _("Ghostscript error:\nProbably the file is not a PostScript file."));
-                        gtk_widget_show(dlg);
-                        gtk_dialog_run(GTK_DIALOG(dlg));
-                        gtk_widget_destroy(dlg);
+                        BonoboArg *arg;
+                        gchar *msg;
+
+                        gtk_window_set_title(GTK_WINDOW(win),
+                                             _("GGV: no document loaded"));
+
+                        arg = bonobo_pbclient_get_value(win->pb, "status",
+                                                        NULL, NULL);
+                        if(arg == NULL)
+                                return;
+                        msg = BONOBO_ARG_GET_STRING(arg);
+                        if(*msg != '\0') {
+                                dlg = gtk_message_dialog_new(GTK_WINDOW(win),
+                                                             GTK_DIALOG_MODAL,
+                                                             GTK_MESSAGE_ERROR,
+                                                             GTK_BUTTONS_OK,
+                                                             msg);
+                                gtk_widget_show(dlg);
+                                gtk_dialog_run(GTK_DIALOG(dlg));
+                                gtk_widget_destroy(dlg);
+                        }
+                        bonobo_arg_release(arg);
                         return;
                 }
         }
-        else if(!g_strcasecmp(event_name, "Bonobo/Property:change:page_count")) {
+        else if(!g_ascii_strcasecmp(event_name, "Bonobo/Property:change:page_count")) {
                 bonobo_ui_component_set_prop(win->uic,
                                              "/commands/FileReload",
                                              "sensitive",
                                              (BONOBO_ARG_GET_LONG(any) > 0)?"1":"0",
                                              NULL);
+                bonobo_ui_component_set_prop(win->uic,
+                                             "/commands/FileClose",
+                                             "sensitive",
+                                             (BONOBO_ARG_GET_LONG(any) > 0)?"1":"0",
+                                             NULL);
         }
 }
 
-static void
-file_sel_unmap(GtkWidget *widget)
-{
-        ggv_get_window_size(widget, &file_sel_width, &file_sel_height);
-}
-
-static gboolean
-file_sel_delete_event(GtkWidget *widget, GdkEventAny *e, gpointer data)
-{
-        gtk_widget_hide(widget);
-        return TRUE;
-}
-
-static void
-file_sel_ok_clicked(GtkWidget *widget, gpointer data)
-{
-        GgvWindow *win = GGV_WINDOW(data);
-        const gchar *fname;
-
-        gtk_widget_hide(win->file_sel);
-        fname = gtk_file_selection_get_filename(GTK_FILE_SELECTION(win->file_sel));
-        if(!ggv_window_load(win, fname)) {
-                GtkWidget *dlg;
-                dlg = gtk_message_dialog_new(GTK_WINDOW(win),
-                                             GTK_DIALOG_MODAL,
-                                             GTK_MESSAGE_ERROR,
-                                             GTK_BUTTONS_OK,
-                                             _("Unable to load file:\n%s"),
-                                             fname);
-                gtk_widget_show(dlg);
-                gtk_dialog_run(GTK_DIALOG(dlg));
-                gtk_widget_destroy(dlg);
-        }
-}
-
-static void
-file_sel_cancel_clicked(GtkWidget *widget, gpointer data)
-{
-        GgvWindow *win = GGV_WINDOW(data);
-
-        gtk_widget_hide(win->file_sel);
-}
-
 static void
 verb_FileOpen(BonoboUIComponent *uic, gpointer data, const char *cname)
 {
         GgvWindow *win = GGV_WINDOW(data);
+        gchar *fname, *escaped_fname;
 
-        if(!win->file_sel) {
-                win->file_sel = gtk_file_selection_new(_("Select a PostScript document"));
-                gtk_window_set_transient_for(GTK_WINDOW(win->file_sel), GTK_WINDOW(win));
-                gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(win->file_sel));
-                gtk_window_set_resizable(GTK_WINDOW(win->file_sel), TRUE);
-                g_signal_connect(G_OBJECT(GTK_FILE_SELECTION(win->file_sel)->ok_button),
-                                 "clicked", G_CALLBACK(file_sel_ok_clicked), win);
-                g_signal_connect(G_OBJECT(GTK_FILE_SELECTION(win->file_sel)->cancel_button),
-                                 "clicked", G_CALLBACK(file_sel_cancel_clicked), win);
-                g_signal_connect(G_OBJECT(win->file_sel), "delete_event",
-                                 G_CALLBACK(file_sel_delete_event), win);
-                g_signal_connect(G_OBJECT(win->file_sel), "unmap",
-                                 G_CALLBACK(file_sel_unmap), NULL);
-        }
-        if(!GTK_WIDGET_VISIBLE(win->file_sel)) {
-                gtk_widget_set_usize(GTK_WIDGET(win->file_sel),
-                                     file_sel_width, file_sel_height);
-                if(win->filename) {
-                        gchar *path = g_path_get_dirname(win->filename);
-			gchar *path2 = g_strconcat(path, "/", NULL);
-			g_free(path);
-                        gtk_file_selection_set_filename(GTK_FILE_SELECTION(win->file_sel), path2);
-                        g_free(path2);
+        fname = ggv_file_sel_request_uri(_("Select a PostScript document"),
+                                         win->filename);
+        if(fname) {
+                if(!ggv_window_load(win, fname)) {
+                        GtkWidget *dlg;
+                        dlg = gtk_message_dialog_new(GTK_WINDOW(win),
+                                                     GTK_DIALOG_MODAL,
+                                                     GTK_MESSAGE_ERROR,
+                                                     GTK_BUTTONS_OK,
+                                                     _("Unable to load file:\n%s"),
+                                                     fname);
+                        gtk_widget_show(dlg);
+                        gtk_dialog_run(GTK_DIALOG(dlg));
+                        gtk_widget_destroy(dlg);
                 }
-                gtk_widget_show(win->file_sel);
-		ggv_raise_and_focus_widget(win->file_sel);
+                g_free(fname);
         }
 }
 
@@ -284,10 +393,21 @@ static void
 verb_FileReload(BonoboUIComponent *uic, gpointer data, const char *cname)
 {
         GgvWindow *win = GGV_WINDOW(data);
-        gint page = win->current_page;
+        GNOME_GGV_PostScriptView ps_view;
+        CORBA_Environment ev;
 
-        ggv_window_load(win, win->filename);
-        bonobo_pbclient_set_long(win->pb, "page", page, NULL);
+        CORBA_exception_init(&ev);
+        ps_view = (GNOME_GGV_PostScriptView)
+                Bonobo_Unknown_queryInterface(win->control,
+                                              "IDL:GNOME/GGV/PostScriptView:1.0",
+                                              &ev);
+        if(!BONOBO_EX(&ev)) {
+                GNOME_GGV_PostScriptView_reload(ps_view, &ev);
+                if(BONOBO_EX(&ev))
+                        g_warning("Could not invoke reload method.");
+                bonobo_object_release_unref(ps_view, &ev);
+        }
+        CORBA_exception_free(&ev);
 }
 
 static void
@@ -295,7 +415,25 @@ verb_FileClose(BonoboUIComponent *uic, gpointer data, const char *cname)
 {
         GgvWindow *win = GGV_WINDOW(data);
 
-        ggv_window_close(win);
+        if(window_list->next != NULL)
+                ggv_window_close(win);
+        else {
+                GNOME_GGV_PostScriptView ps_view;
+                CORBA_Environment ev;
+
+                CORBA_exception_init(&ev);
+                ps_view = (GNOME_GGV_PostScriptView)
+                        Bonobo_Unknown_queryInterface(win->control,
+                                                      "IDL:GNOME/GGV/PostScriptView:1.0",
+                                                      &ev);
+                if(!BONOBO_EX(&ev)) {
+                        GNOME_GGV_PostScriptView_close(ps_view, &ev);
+                        if(BONOBO_EX(&ev))
+                                g_warning("Could not invoke reload method.");
+                        bonobo_object_release_unref(ps_view, &ev);
+                }
+                CORBA_exception_free(&ev);                
+        }
 }
 
 static void
@@ -305,18 +443,48 @@ verb_FileNew(BonoboUIComponent *uic, gpointer data, const char *cname)
         gtk_widget_show(win);
 }
 
+void
+ggv_window_destroy_all()
+{
+        while(window_list)
+                gtk_widget_destroy(GTK_WIDGET(window_list->data));
+}
+
 static void
 verb_FileExit(BonoboUIComponent *uic, gpointer data, const char *cname)
 {
         GList *l;
 
-        for(l = window_list; l != NULL; l = l->next)
-                gtk_widget_destroy(GTK_WIDGET(l->data));
-        g_list_free(window_list);
-        window_list = NULL;
+        ggv_window_destroy_all();
         bonobo_main_quit();
 }
 
+static void
+listener_ViewFullscreen (BonoboUIComponent *uic, const char *path,
+                         Bonobo_UIComponent_EventType type, const char *state,
+                         gpointer user_data)
+{
+	GgvWindow *window;
+        gboolean state_f;
+
+	g_return_if_fail(user_data != NULL);
+	g_return_if_fail(GGV_IS_WINDOW (user_data));
+
+	if(type != Bonobo_UIComponent_STATE_CHANGED)
+		return;
+
+	if(!state)
+		return;
+
+        window = GGV_WINDOW(user_data);
+        state_f = atoi(state);
+
+        if(!strcmp(path, "ViewFullscreen")) {
+                if(window->fullscreen != state_f)
+                        ggv_window_set_fullscreen(window, state_f);
+        }
+}
+
 static void
 listener_SettingsShow (BonoboUIComponent *uic, const char *path,
                        Bonobo_UIComponent_EventType type, const char *state,
@@ -391,6 +559,14 @@ verb_HelpAbout(BonoboUIComponent *uic, gpointer data, const char *cname)
                 "And many more...",
  		NULL
 	};
+
+	static const char *documenters[] = {
+		NULL
+	};
+
+	/* Translator Credits */
+	char *translator_credits = _("translator_credits");
+
         static GdkPixbuf *logo = NULL;
 
         if(!logo)
@@ -404,8 +580,8 @@ verb_HelpAbout(BonoboUIComponent *uic, gpointer data, const char *cname)
 			_("Copyright (C) 1998-2002 The Free Software Foundation"),
 			_("The GNOME PostScript document previewer"),
 			authors,
-			NULL, /* char **documentors */
-			NULL, /* char *translators */
+			documenters,
+			strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
 			logo);
                 gtk_window_set_transient_for(GTK_WINDOW(about),
                                              GTK_WINDOW(data));
@@ -430,15 +606,16 @@ verb_DnDNewWindow(BonoboUIComponent *uic, gpointer user_data, const char *cname)
 {
         GgvWindow *win = GGV_WINDOW(user_data);
         gchar **uri;
+        GtkWidget *newwin;
 
         uri = win->uris_to_open;
-        while(*uri) {
-                GtkWidget *newwin;
-                if(!g_strncasecmp("file:", *uri, 5)) {
-                        newwin = ggv_window_new();
-                        gtk_widget_show(newwin);
-                        ggv_window_load(GGV_WINDOW(newwin), (*uri) + 5);
-                }
+        while(*uri && **uri != '\0') {
+#ifdef _DEBUG
+                g_message("URI %s", *uri);
+#endif /* _DEBUG */
+                newwin = ggv_window_new();
+                gtk_widget_show(newwin);
+                ggv_window_load(GGV_WINDOW(newwin), (*uri));
                 uri++;
         }
         g_strfreev(win->uris_to_open);
@@ -454,15 +631,16 @@ verb_DnDThisWindow(BonoboUIComponent *uic, gpointer user_data, const char *cname
 
         uri = win->uris_to_open;
         newwin = GTK_WIDGET(win);
-        while(*uri) {
-                if(!g_strncasecmp("file:", *uri, 5)) {
-                        if(newwin == NULL) {
-                                newwin = ggv_window_new();
-                                gtk_widget_show(newwin);
-                        }
-                        ggv_window_load(GGV_WINDOW(newwin), (*uri) + 5);
-                        newwin = NULL;
+        while(*uri && **uri != '\0') {
+#ifdef _DEBUG
+                g_message("URI %s", *uri);
+#endif /* _DEBUG */
+                if(newwin == NULL) {
+                        newwin = ggv_window_new();
+                        gtk_widget_show(newwin);
                 }
+                ggv_window_load(GGV_WINDOW(newwin), (*uri));
+                newwin = NULL;
                 uri++;
         }
         g_strfreev(win->uris_to_open);
@@ -497,15 +675,12 @@ control_frame_activate_uri(BonoboControlFrame *control_frame, const char *uri,
                            gboolean relative, gpointer data)
 {
 	GgvWindow *win;
-	char *path;
 
 	g_return_if_fail(uri != NULL);
 
-	if(g_strncasecmp("file:", uri, 5) == 0) {
-                win = GGV_WINDOW(ggv_window_new());
-                ggv_window_load(win, uri + 5);
-                gtk_widget_show(GTK_WIDGET(win));
-        }
+        win = GGV_WINDOW(ggv_window_new());
+        ggv_window_load(win, uri);
+        gtk_widget_show(GTK_WIDGET(win));
 }
 
 static void
@@ -543,6 +718,8 @@ ggv_window_destroy(GtkObject *object)
                 win->filename = NULL;
         }
 
+        window_list = g_list_remove(window_list, win);
+
         ggv_get_window_size(GTK_WIDGET(object),
                             &ggv_default_width, &ggv_default_height);
 
@@ -557,6 +734,66 @@ ggv_window_delete_event(GtkWidget *widget, GdkEventAny *e)
         return TRUE;
 }
 
+static void
+prefs_changed(GConfClient* client, guint cnxn_id,
+              GConfEntry *entry, gpointer user_data)
+{
+        if(!strcmp(entry->key, "/desktop/gnome/interface/toolbar_style") &&
+           ggv_toolbar_style == GGV_TOOLBAR_STYLE_DEFAULT) {
+                sync_toolbar_style(GGV_WINDOW(user_data));
+        }
+        else if(!strcmp(entry->key, "/apps/ggv/layout/toolbarstyle")) {
+                ggv_toolbar_style = gconf_value_get_int(entry->value);
+                sync_toolbar_style(GGV_WINDOW(user_data));
+        }
+        else if(!strcmp(entry->key, "/apps/ggv/layout/showtoolbar")) {
+                GgvWindow *window = GGV_WINDOW(user_data);
+                gboolean state_f = gconf_value_get_bool(entry->value);
+                if(window->show_toolbar != state_f) {
+                        window->show_toolbar = state_f;
+                        sync_settings_menu_items(window);
+                        sync_settings_popup_items(window);
+                        bonobo_ui_component_set_prop(window->uic, "/Toolbar", "hidden",
+                                                     state_f?"0":"1", NULL);
+                }
+        }
+        else if(!strcmp(entry->key, "/apps/ggv/layout/showmenubar")) {
+                GgvWindow *window = GGV_WINDOW(user_data);
+                gboolean state_f = gconf_value_get_bool(entry->value);
+                if(window->show_menus != state_f) {
+                        window->show_menus = state_f;
+                        sync_settings_menu_items(window);
+                        sync_settings_popup_items(window);
+                        bonobo_ui_component_set_prop(window->uic, "/menu", "hidden",
+                                                     state_f?"0":"1", NULL);
+                }
+        }
+        else if(!strcmp(entry->key, "/apps/ggv/layout/showpanel")) {
+                GgvWindow *window = GGV_WINDOW(user_data);
+                gboolean state_f = gconf_value_get_bool(entry->value);
+                if(window->show_sidebar != state_f) {
+                        window->show_sidebar = state_f;
+                        sync_settings_menu_items(window);
+                        sync_settings_popup_items(window);
+                        bonobo_ui_component_set_prop(window->uic, "/Sidebar", "hidden",
+                                                     state_f?"0":"1", NULL);
+                }
+        }
+        else if(!strcmp(entry->key, "/apps/ggv/layout/showstatusbar")) {
+                GgvWindow *window = GGV_WINDOW(user_data);
+                gboolean state_f = gconf_value_get_bool(entry->value);
+                if(window->show_statusbar != state_f) {
+                        window->show_statusbar = state_f;
+                        sync_settings_menu_items(window);
+                        sync_settings_popup_items(window);
+                        if(state_f)
+                                gtk_widget_show(window->statusbar);
+                        else
+                                gtk_widget_hide(window->statusbar);
+                }
+        }
+}
+
 static void
 ggv_window_class_init(GgvWindowClass *class)
 {
@@ -615,13 +852,11 @@ ggv_window_new(void)
         GgvWindow *win;
 	BonoboUIContainer *ui_container;
 	BonoboUIComponent *uic, *popup_uic;
-	GdkGeometry geometry;
         CORBA_Environment ev;
         Bonobo_Control control;
-        GtkWidget *widget, *sw, *hbox, *image;
+        GtkWidget *widget;
         Bonobo_PropertyBag pb;
         gchar *mask;
-        GtkTreeSelection *sel;
 
 	static const GtkTargetEntry drag_types[] = {
 		{ "text/uri-list", 0, TARGET_URI_LIST }
@@ -632,12 +867,14 @@ ggv_window_new(void)
         control = bonobo_get_object(GGV_POSTSCRIPT_VIEWER_CONTROL_IID,
                                     "Bonobo/Control", &ev);
         if(BONOBO_EX(&ev) || control == CORBA_OBJECT_NIL) {
+                g_warning("Could not get GGV control: '%s'",
+                          bonobo_exception_get_text(&ev));
                 CORBA_exception_free(&ev);
                 return NULL;
         }
         CORBA_exception_free(&ev);
 
-	win = GGV_WINDOW(g_object_new(GGV_TYPE_WINDOW, "win_name", "ggv", "title", _("Gnome Ghostview"), NULL));
+	win = GGV_WINDOW(g_object_new(GGV_TYPE_WINDOW, "win_name", "ggv", "title", _("GGV: no document loaded"), NULL));
         win->control = control;
 
         win->current_page = -2;
@@ -679,7 +916,7 @@ ggv_window_new(void)
         win->uic = uic;
 	bonobo_ui_component_set_container(uic, BONOBO_OBJREF(ui_container),
                                           NULL);
-	bonobo_ui_util_set_ui(uic, NULL, "ggv-ui.xml", "GGV", NULL);
+	bonobo_ui_util_set_ui(uic, DATADIR, "ggv-ui.xml", "GGV", NULL);
         sync_toolbar_style(win);
 	bonobo_ui_component_add_verb_list_with_data(uic, ggv_app_verbs, win);
 	bonobo_ui_component_add_listener(uic, "SettingsShowMenus",
@@ -690,6 +927,8 @@ ggv_window_new(void)
                                          listener_SettingsShow, win);
 	bonobo_ui_component_add_listener(uic, "SettingsShowStatusbar",
                                          listener_SettingsShow, win);
+	bonobo_ui_component_add_listener(uic, "ViewFullscreen",
+                                         listener_ViewFullscreen, win);
         sync_settings_menu_items(win);
 
         if(!win->show_toolbar)
@@ -727,6 +966,11 @@ ggv_window_new(void)
                                              "sensitive",
                                              (bonobo_pbclient_get_long(pb, "page", NULL) >= 0)?"1":"0",
                                              NULL);
+                bonobo_ui_component_set_prop(win->uic,
+                                             "/commands/FileClose",
+                                             "sensitive",
+                                             (bonobo_pbclient_get_long(pb, "page", NULL) >= 0)?"1":"0",
+                                             NULL);
                 bonobo_event_source_client_add_listener(pb,
                                                         (BonoboListenerCallbackFn)control_property_changed_handler,
                                                         mask, NULL, win);
@@ -739,7 +983,7 @@ ggv_window_new(void)
                 g_warning("Control does not have a popup component.");
         }
         else {
-                bonobo_ui_util_set_ui(popup_uic, NULL, "ggv-ui.xml",
+                bonobo_ui_util_set_ui(popup_uic, DATADIR, "ggv-ui.xml",
                                       "GGV", NULL);
                 bonobo_ui_component_add_listener(popup_uic, "SettingsShowMenus",
                                                  listener_SettingsShow, win);
@@ -749,6 +993,8 @@ ggv_window_new(void)
                                                  listener_SettingsShow, win);
                 bonobo_ui_component_add_listener(popup_uic, "SettingsShowStatusbar",
                                                  listener_SettingsShow, win);
+                bonobo_ui_component_add_listener(popup_uic, "ViewFullscreen",
+                                                 listener_ViewFullscreen, win);
                 win->popup_uic = popup_uic;
                 sync_settings_popup_items(win);
         }
@@ -758,6 +1004,15 @@ ggv_window_new(void)
                              ggv_default_width, ggv_default_height);
         gtk_window_set_policy(GTK_WINDOW(win), TRUE, TRUE, FALSE);
 
+        gconf_client_notify_add(ggv_prefs_gconf_client(),
+                                "/desktop/gnome/interface/toolbar_style",
+                                (GConfClientNotifyFunc)prefs_changed,
+                                win, NULL, NULL);
+        gconf_client_notify_add(ggv_prefs_gconf_client(),
+                                "/apps/ggv/layout",
+                                (GConfClientNotifyFunc)prefs_changed,
+                                win, NULL, NULL);
+
 	return GTK_WIDGET(win);
 }
 
@@ -766,16 +1021,12 @@ ggv_window_load(GgvWindow *win, const gchar *filename)
 {
         CORBA_Environment ev;
         Bonobo_PersistFile pf;
-        gchar *full_path;
+        gchar *title, *unescaped;
 
         g_return_val_if_fail(win != NULL, FALSE);
         g_return_val_if_fail(GGV_IS_WINDOW(win), FALSE);
         g_return_val_if_fail(filename != NULL, FALSE);
 
-        /* TODO: add handling of stdin: should probably be solved by 
-           copying stdin to a stream. */
-        g_assert(strcmp(filename, "-"));
-
         CORBA_exception_init(&ev);
         pf = Bonobo_Unknown_queryInterface(win->control,
                                            "IDL:Bonobo/PersistFile:1.0", &ev);
@@ -784,27 +1035,18 @@ ggv_window_load(GgvWindow *win, const gchar *filename)
                 return FALSE;
         }
 
-        /* we need to pass full path to file to the control as we
-           do not really know where it resides.
-           NOTE: this still only works when the control is running
-           on the same computer as the GGV. I suppose this will mostly
-           be the case ;) */
-        if(*filename != '/') {
-                gchar *cwd;
-
-                cwd = g_get_current_dir();
-                full_path = g_strconcat(cwd, "/", filename, NULL);
-                g_free(cwd);
-        }
-        else
-                full_path = g_strdup(filename);
-
-        Bonobo_PersistFile_load(pf, full_path, &ev);
+        Bonobo_PersistFile_load(pf, filename, &ev);
         bonobo_object_release_unref(pf, NULL);
 
         if(win->filename)
                 g_free(win->filename);
-        win->filename = full_path;
+        win->filename = g_strdup(filename);
+
+        unescaped = gnome_vfs_unescape_string_for_display(win->filename);
+        title = g_strconcat(_("GGV: "), unescaped, NULL);
+        g_free(unescaped);
+        gtk_window_set_title(GTK_WINDOW(win), title);
+        g_free(title);
 
         if(BONOBO_EX(&ev)) {
                 CORBA_exception_free(&ev);
@@ -821,8 +1063,6 @@ ggv_window_close(GgvWindow *win)
         g_return_if_fail(win != NULL);
         g_return_if_fail(GGV_IS_WINDOW(win));
 
-        window_list = g_list_remove(window_list, win);
-
         gtk_widget_destroy(GTK_WIDGET(win));
 
         if(!window_list)
